@@ -3,7 +3,7 @@
 /*  bam_ampliconclip.c -- loads amplicon primers from a BED file and cuts reads
                           from the 5' end.
 
-    Copyright (C) 2020-2022 Genome Research Ltd.
+    Copyright (C) 2020-2023 Genome Research Ltd.
 
     Authors: Andrew Whitwham <aw7@sanger.ac.uk>
              Rob Davies <rmd+git@sanger.ac.uk>
@@ -61,6 +61,7 @@ typedef struct {
     int oa_tag;
     int del_tag;
     int tol;
+    int unmap_len;
     char *arg_list;
     char *stats_file;
     char *rejects_file;
@@ -640,6 +641,7 @@ static int bam_clip(samFile *in, samFile *out, samFile *reject, char *bedfile,
     long filtered = 0, written = 0, failed = 0;
     kstring_t str = KS_INITIALIZE;
     kstring_t oat = KS_INITIALIZE;
+    kstring_t seq = KS_INITIALIZE;
     bed_entry_list_t *sites;
     FILE *stats_fp = samtools_stderr;
     khash_t(bed_list_hash) *bed_hash = kh_init(bed_list_hash);
@@ -831,16 +833,46 @@ static int bam_clip(samFile *in, samFile *out, samFile *reject, char *bedfile,
                 }
             }
 
-            if (param->fail_len >= 0 || param->filter_len >= 0) {
-               hts_pos_t aql = active_query_len(b);
+            if (param->fail_len >= 0 || param->filter_len >= 0 || param->unmap_len >= 0) {
+                hts_pos_t aql = active_query_len(b);
 
-               if (param->fail_len >= 0 && aql <= param->fail_len) {
-                   b->core.flag |= BAM_FQCFAIL;
-               }
+                if (param->fail_len >= 0 && aql <= param->fail_len) {
+                    b->core.flag |= BAM_FQCFAIL;
+                }
 
-               if (param->filter_len >= 0 && aql <= param->filter_len) {
-                   filter = 1;
-               }
+                if (param->filter_len >= 0 && aql <= param->filter_len) {
+                    filter = 1;
+                }
+
+                if (param->unmap_len >= 0 && aql <= param->unmap_len) {
+
+                    if (ks_resize(&seq, b->core.l_qseq) < 0) {
+                        fprintf(samtools_stderr, "[ampliconclip] error: allocate memory for sequence %s\n", bam_get_seq(b));
+                        goto fail;
+                    }
+
+                    ks_clear(&seq);
+                    char *sb = ks_str(&seq);
+                    uint8_t *sequence = bam_get_seq(b);
+                    int i;
+
+                    for (i = 0; i < b->core.l_qseq ; ++i) {
+                        *sb++ = seq_nt16_str[bam_seqi(sequence, i)];
+                    }
+
+                    if (bam_set1(b_tmp, b->core.l_qname - b->core.l_extranul - 1, bam_get_qname(b),
+                                 (b->core.flag | BAM_FUNMAP), b->core.tid, b->core.pos, 0,
+                                 0, NULL, b->core.mtid, b->core.mpos, b->core.isize,
+                                 b->core.l_qseq, seq.s, (const char *)bam_get_qual(b),
+                                 bam_get_l_aux(b)) < 0) {
+                        fprintf(samtools_stderr, "[ampliconclip] error: could not unmap read %s\n", bam_get_seq(b));
+                        goto fail;
+                    }
+
+                    memcpy(bam_get_aux(b_tmp), bam_get_aux(b), bam_get_l_aux(b));
+                    b_tmp->l_data += bam_get_l_aux(b);
+                    swap_bams(&b, &b_tmp);
+                }
            }
 
            if (b->core.flag & BAM_FQCFAIL) {
@@ -915,6 +947,7 @@ static int bam_clip(samFile *in, samFile *out, samFile *reject, char *bedfile,
 fail:
     destroy_bed_hash(bed_hash);
     ks_free(&oat);
+    ks_free(&seq);
     sam_hdr_destroy(header);
     bam_destroy1(b);
     bam_destroy1(b_tmp);
@@ -937,6 +970,7 @@ static void usage(void) {
     fprintf(samtools_stderr, " --fail              mark unclipped, mapped reads as QCFAIL.\n");
     fprintf(samtools_stderr, " --filter-len INT    do not output reads INT size or shorter.\n");
     fprintf(samtools_stderr, " --fail-len   INT    mark as QCFAIL reads INT size or shorter.\n");
+    fprintf(samtools_stderr, " --unmap-len  INT    unmap reads INT size or shorter, default 0.\n");
     fprintf(samtools_stderr, " --no-excluded       do not write excluded reads (unmapped or QCFAIL).\n");
     fprintf(samtools_stderr, " --rejects-file FILE file to write filtered reads.\n");
     fprintf(samtools_stderr, " --original          for clipped entries add an OA tag with original data.\n");
@@ -957,7 +991,7 @@ int amplicon_clip_main(int argc, char **argv) {
     htsThreadPool p = {NULL, 0};
     samFile *in = NULL, *out = NULL, *reject = NULL;
     clipping_type clipping = soft_clip;
-    cl_param_t param = {1, 0, 0, 0, 0, -1, -1, 0, 0, 1, 5, NULL, NULL, NULL};
+    cl_param_t param = {1, 0, 0, 0, 0, -1, -1, 0, 0, 1, 5, 0, NULL, NULL, NULL};
 
     static const struct option lopts[] = {
         SAM_OPT_GLOBAL_OPTIONS('-', 0, 'O', 0, 0, '@'),
@@ -975,6 +1009,7 @@ int amplicon_clip_main(int argc, char **argv) {
         {"original", no_argument, NULL, 1013},
         {"keep-tag", no_argument, NULL, 1014},
         {"tolerance", required_argument, NULL, 1015},
+        {"unmap-len", required_argument, NULL, 1016},
         {NULL, 0, NULL, 0}
     };
 
@@ -998,6 +1033,7 @@ int amplicon_clip_main(int argc, char **argv) {
             case 1013: param.oa_tag = 1; break;
             case 1014: param.del_tag = 0; break;
             case 1015: param.tol = atoi(optarg); break;
+            case 1016: param.unmap_len = atoi(optarg); break;
             default:  if (parse_sam_global_opt(c, optarg, lopts, &ga) == 0) break;
                       /* else fall-through */
             case '?': usage(); samtools_exit(1);
@@ -1016,7 +1052,7 @@ int amplicon_clip_main(int argc, char **argv) {
 
     if (param.tol < 0) {
         fprintf(samtools_stderr, "[ampliconclip] warning: invalid tolerance of %d,"
-                        " reseting tolerance to default of 5.\n", param.tol);
+                        " resetting tolerance to default of 5.\n", param.tol);
         param.tol = 5;
     }
 
