@@ -2,7 +2,7 @@
 
 /*  sam_view.c -- SAM<->BAM<->CRAM conversion.
 
-    Copyright (C) 2009-2022 Genome Research Ltd.
+    Copyright (C) 2009-2023 Genome Research Ltd.
     Portions copyright (C) 2009, 2011, 2012 Broad Institute.
 
     Author: Heng Li <lh3@sanger.ac.uk>
@@ -45,13 +45,11 @@ DEALINGS IN THE SOFTWARE.  */
 #include "sam_opts.h"
 #include "bam.h" // for bam_get_library and bam_remove_B
 #include "bedidx.h"
+#include "sam_utils.h"
 
 KHASH_SET_INIT_STR(str)
 typedef khash_t(str) *strhash_t;
 
-KHASH_SET_INIT_INT(aux_exists)
-typedef khash_t(aux_exists) *auxhash_t;
-
 // This structure contains the settings for a samview run
 typedef struct samview_settings {
     strhash_t rghash;
@@ -91,6 +89,8 @@ typedef struct samview_settings {
     char *fn_in, *fn_idx_in, *fn_out, *fn_fai, *fn_un_out, *fn_out_idx, *fn_un_out_idx;
     int fetch_pairs, nreglist;
     hts_reglist_t *reglist;
+    int sanitize;
+    int count_rf; // CRAM_OPT_REQUIRED_FIELDS for view -c
 } samview_settings_t;
 
 // Copied from htslib/sam.c.
@@ -141,11 +141,17 @@ static inline uint8_t *skip_aux(uint8_t *s, uint8_t *end)
     }
 }
 
-// Returns 0 to indicate read should be output 1 otherwise
+// Returns 0 to indicate read should be output 1 otherwise,
+// and -1 on error.
 static int process_aln(const sam_hdr_t *h, bam1_t *b, samview_settings_t* settings)
 {
-    if (settings->filter && sam_passes_filter(h, b, settings->filter) < 1)
-        return 1;
+    if (settings->filter) {
+        int r = sam_passes_filter(h, b, settings->filter);
+        if (r < 0)  // err
+            return -1;
+        if (r == 0) // filter-out
+            return 1;
+    }
 
     if (settings->remove_B) bam_remove_B(b);
     if (settings->min_qlen > 0) {
@@ -394,33 +400,6 @@ static inline void change_flag(bam1_t *b, samview_settings_t *settings)
         b->core.flag &= ~settings->remove_flag;
 }
 
-int parse_aux_list(auxhash_t *h, char *optarg) {
-    if (!*h)
-        *h = kh_init(aux_exists);
-
-    while (strlen(optarg) >= 2) {
-        int x = optarg[0]<<8 | optarg[1];
-        int ret = 0;
-        kh_put(aux_exists, *h, x, &ret);
-        if (ret < 0)
-            return -1;
-
-        optarg += 2;
-        if (*optarg == ',') // allow white-space too for easy `cat file`?
-            optarg++;
-        else if (*optarg != 0)
-            break;
-    }
-
-    if (strlen(optarg) != 0) {
-        fprintf(samtools_stderr, "main_samview: Error parsing option, "
-                "auxiliary tags should be exactly two characters long.\n");
-        return -1;
-    }
-
-    return 0;
-}
-
 static int cmp_reglist_intervals(const void *aptr, const void *bptr)
 {
     hts_pair_pos_t *a = (hts_pair_pos_t*)aptr;
@@ -610,7 +589,9 @@ static int fetch_pairs_collect_mates(samview_settings_t *conf, hts_itr_multi_t *
     while ((r =sam_itr_multi_next(conf->in, iter, rec))>=0) {
         if ( (rec->core.flag & BAM_FPAIRED) == 0 ) continue;
         if ( rec->core.mtid>=0 && bed_overlap(conf->bed, sam_hdr_tid2name(conf->header,rec->core.mtid), rec->core.mpos, rec->core.mpos) ) continue;
-        if ( process_aln(conf->header, rec, conf) ) continue;
+        int p = process_aln(conf->header, rec, conf);
+        if (p < 0)  goto out;
+        if (p == 1) continue;
 
         nmates++;
 
@@ -661,13 +642,16 @@ static int fetch_pairs_collect_mates(samview_settings_t *conf, hts_itr_multi_t *
              k = kh_get(names,mate_names,bam_get_qname(rec));
              if ( k != kh_end(mate_names) ) drop = 0;
         }
-        if (!drop && process_aln(conf->header, rec, conf) == 0) {
+        int p = 0;
+        if (!drop && (p=process_aln(conf->header, rec, conf))== 0) {
             if (adjust_tags(conf->header, rec, conf) != 0)
                 goto out;
             if (check_sam_write1(conf->out, conf->header, rec, conf->fn_out,
                                  &write_error) < 0)
                 goto out;
         }
+        if (p < 0)
+            goto out;
     }
 
     if (r < -1) {
@@ -694,7 +678,16 @@ static int fetch_pairs_collect_mates(samview_settings_t *conf, hts_itr_multi_t *
 // Common code for processing and writing a record
 static inline int process_one_record(samview_settings_t *conf, bam1_t *b,
                                      int *write_error) {
-    if (!process_aln(conf->header, b, conf)) {
+    if (conf->sanitize)
+        if (bam_sanitize(conf->header, b, conf->sanitize) < 0)
+            return -1;
+
+    int p;
+    if ((p = process_aln(conf->header, b, conf)) < 0) {
+        // error
+        return -1;
+    } else if (p == 0) {
+        // emit read
         if (!conf->is_count) {
             change_flag(b, conf);
             if (adjust_tags(conf->header, b, conf) != 0)
@@ -735,17 +728,17 @@ static inline int process_one_record(samview_settings_t *conf, bam1_t *b,
 
 static int stream_view(samview_settings_t *conf) {
     bam1_t *b = bam_init1();
-    int write_error = 0, r;
+    int write_error = 0, r, p = 0;
     if (!b) {
         print_error_errno("view", "could not allocate bam record");
         return 1;
     }
     errno = 0; // prevent false error messages.
     while ((r = sam_read1(conf->in, conf->header, b)) >= 0) {
-        if (process_one_record(conf, b, &write_error) < 0) break;
+        if ((p = process_one_record(conf, b, &write_error)) < 0) break;
     }
     bam_destroy1(b);
-    if (r < -1) {
+    if (r < -1 || p < 0) {
         print_error_errno("view", "error reading file \"%s\"", conf->fn_in);
         return 1;
     }
@@ -786,6 +779,13 @@ static int is_sam(const char *fn) {
     return (l >= 4 && strcasecmp(fn + l-4, ".sam") == 0);
 }
 
+static void aux_list_free(samview_settings_t *settings) {
+    if (settings->keep_tag)
+        kh_destroy(aux_exists, settings->keep_tag);
+    if (settings->remove_tag)
+        kh_destroy(aux_exists, settings->remove_tag);
+}
+
 int main_samview(int argc, char *argv[])
 {
     samview_settings_t settings;
@@ -799,6 +799,7 @@ int main_samview(int argc, char *argv[])
 
     memset(&settings,0,sizeof(settings));
     settings.subsam_frac = -1.0;
+    settings.count_rf = SAM_FLAG; // don't want 0, and this is quick
 
     static const struct option lopts[] = {
         SAM_OPT_GLOBAL_OPTIONS('-', 0, 'O', 0, 'T', '@'),
@@ -853,6 +854,7 @@ int main_samview(int argc, char *argv[])
         {"unoutput", required_argument, NULL, 'U'},
         {"use-index", no_argument, NULL, 'M'},
         {"with-header", no_argument, NULL, 'h'},
+        {"sanitize", required_argument, NULL, 'z'},
     };
 
     /* parse command-line options */
@@ -869,7 +871,7 @@ int main_samview(int argc, char *argv[])
 
     char *tmp;
     while ((c = getopt_long(argc, argv,
-                            "SbBcCt:h1Ho:O:q:f:F:G:ul:r:T:R:N:d:D:L:s:@:m:x:U:MXe:pP",
+                            "SbBcCt:h1Ho:O:q:f:F:G:ul:r:T:R:N:d:D:L:s:@:m:x:U:MXe:pPz:",
                             lopts, NULL)) >= 0) {
         switch (c) {
         case 's':
@@ -885,6 +887,7 @@ int main_samview(int argc, char *argv[])
                 print_error("view", "Incorrect sampling argument \"%s\"", optarg);
                 goto view_end;
             }
+            settings.count_rf |= SAM_QNAME;
             break;
         case LONGOPT('s'):
             settings.subsam_frac = strtod(optarg, &tmp);
@@ -892,9 +895,13 @@ int main_samview(int argc, char *argv[])
                 print_error("view", "Incorrect sampling argument \"%s\"", optarg);
                 goto view_end;
             }
+            settings.count_rf |= SAM_QNAME;
             break;
         case LONGOPT('S'): settings.subsam_seed = atoi(optarg); break;
-        case 'm': settings.min_qlen = atoi(optarg); break;
+        case 'm':
+            settings.min_qlen = atoi(optarg);
+            settings.count_rf |= SAM_SEQ;
+            break;
         case 'c': settings.is_count = 1; break;
         case 'S': break;
         case 'b': out_format = "b"; break;
@@ -906,17 +913,40 @@ int main_samview(int argc, char *argv[])
         case 'o': settings.fn_out = strdup(optarg); break;
         case 'U': settings.fn_un_out = strdup(optarg); break;
         case 'X': has_index_file = 1; break;
-        case 'f': settings.flag_on |= bam_str2flag(optarg); break;
-        case 'F': settings.flag_off |= bam_str2flag(optarg); break;
+        case 'f':
+            settings.flag_on |= bam_str2flag(optarg);
+            settings.count_rf |= SAM_FLAG | SAM_RNEXT;
+            break;
+        case 'F':
+            settings.flag_off |= bam_str2flag(optarg);
+            settings.count_rf |= SAM_FLAG | SAM_RNEXT;
+            break;
         case LONGOPT('g'):
-            settings.flag_anyon |= bam_str2flag(optarg); break;
-        case 'G': settings.flag_alloff |= bam_str2flag(optarg); break;
-        case 'q': settings.min_mapQ = atoi(optarg); break;
+            settings.flag_anyon |= bam_str2flag(optarg);
+            settings.count_rf |= SAM_FLAG | SAM_RNEXT;
+            break;
+        case 'G':
+            settings.flag_alloff |= bam_str2flag(optarg);
+            settings.count_rf |= SAM_FLAG | SAM_RNEXT;
+            break;
+        case 'q':
+            settings.min_mapQ = atoi(optarg);
+            settings.count_rf |= SAM_MAPQ;
+            break;
         case 'u': compress_level = 0; break;
         case '1': compress_level = 1; break;
-        case 'l': settings.library = strdup(optarg); break;
+        case 'l':
+            settings.library = strdup(optarg);
+            settings.count_rf |= SAM_RGAUX;
+            break;
         case 'p': settings.unmap = 1; break;
         case 'P': settings.fetch_pairs = 1; settings.multi_region = 1; break;
+        case 'z':
+            if ((settings.sanitize = bam_sanitize_options(optarg)) < 0) {
+                ret = 1;
+                goto view_end;
+            }
+            break;
         case LONGOPT('L'):
             settings.multi_region = 1;
             // fall through
@@ -926,28 +956,33 @@ int main_samview(int argc, char *argv[])
                 ret = 1;
                 goto view_end;
             }
+            settings.count_rf |= SAM_POS | SAM_RNAME | SAM_CIGAR;
             break;
         case 'r':
             if (add_read_group_single("view", &settings, optarg) != 0) {
                 ret = 1;
                 goto view_end;
             }
+            settings.count_rf |= SAM_RGAUX;
             break;
         case 'R':
             if (add_read_groups_file("view", &settings, optarg) != 0) {
                 ret = 1;
                 goto view_end;
             }
+            settings.count_rf |= SAM_RGAUX;
             break;
         case 'N':
             if (add_read_names_file("view", &settings, optarg) != 0) {
                 ret = 1;
                 goto view_end;
             }
+            settings.count_rf |= SAM_QNAME;
             break;
+
         case 'd':
             if (strlen(optarg) < 2 || (strlen(optarg) > 2 && optarg[2] != ':')) {
-                print_error_errno("view", "Invalid \"tag:value\" option: \"%s\"", optarg);
+                print_error("view", "Invalid \"tag:value\" option: \"%s\"", optarg);
                 ret = 1;
                 goto view_end;
             }
@@ -972,13 +1007,22 @@ int main_samview(int argc, char *argv[])
                 ret = 1;
                 goto view_end;
             }
+            // Some tag filtering affects other fields
+            if (memcmp(settings.tag, "NM", 2) == 0 ||
+                memcmp(settings.tag, "MD", 2) == 0)
+                settings.count_rf |= SAM_AUX | SAM_SEQ;
+            else if (memcmp(settings.tag, "RG", 2) == 0)
+                settings.count_rf |= SAM_RGAUX;
+            else
+                settings.count_rf |= SAM_AUX;
             break;
+
         case 'D':
             // Allow ";" as delimiter besides ":" to support MinGW CLI POSIX
             // path translation as described at:
             // http://www.mingw.org/wiki/Posix_path_conversion
             if (strlen(optarg) < 4 || (optarg[2] != ':' && optarg[2] != ';')) {
-                print_error_errno("view", "Invalid \"tag:file\" option: \"%s\"", optarg);
+                print_error("view", "Invalid \"tag:file\" option: \"%s\"", optarg);
                 ret = 1;
                 goto view_end;
             }
@@ -1002,7 +1046,16 @@ int main_samview(int argc, char *argv[])
                 ret = 1;
                 goto view_end;
             }
+            // Some tag filtering affects other fields
+            if (memcmp(settings.tag, "NM", 2) == 0 ||
+                memcmp(settings.tag, "MD", 2) == 0)
+                settings.count_rf |= SAM_AUX | SAM_SEQ;
+            else if (memcmp(settings.tag, "RG", 2) == 0)
+                settings.count_rf |= SAM_RGAUX;
+            else
+                settings.count_rf |= SAM_AUX;
             break;
+
         case LONGOPT('?'):
             return usage(samtools_stdout, EXIT_SUCCESS, 1);
         case '?':
@@ -1031,23 +1084,30 @@ int main_samview(int argc, char *argv[])
                 print_error("main_samview", "Couldn't initialise filter");
                 return 1;
             }
+            settings.count_rf = INT_MAX; // no way to know what we need
             break;
         case LONGOPT('r'): settings.remove_flag |= bam_str2flag(optarg); break;
         case LONGOPT('a'): settings.add_flag |= bam_str2flag(optarg); break;
 
         case 'x':
             if (*optarg == '^') {
-                if (parse_aux_list(&settings.keep_tag, optarg+1))
+                if (parse_aux_list(&settings.keep_tag, optarg+1, "main_samview")) {
+                    aux_list_free(&settings);
                     return usage(samtools_stderr, EXIT_FAILURE, 0);
+                }
             } else {
-                if (parse_aux_list(&settings.remove_tag, optarg))
+                if (parse_aux_list(&settings.remove_tag, optarg, "main_samview")) {
+                    aux_list_free(&settings);
                     return usage(samtools_stderr, EXIT_FAILURE, 0);
+                }
             }
             break;
 
         case LONGOPT('x'):
-            if (parse_aux_list(&settings.keep_tag, optarg))
+            if (parse_aux_list(&settings.keep_tag, optarg, "main_samview")) {
+                aux_list_free(&settings);
                 return usage(samtools_stderr, EXIT_FAILURE, 0);
+            }
             break;
 
         default:
@@ -1223,7 +1283,7 @@ int main_samview(int argc, char *argv[])
         settings.unmap = 0;  // Not valid in counting mode
     }
 
-    if (ga.nthreads > 1) {
+    if (ga.nthreads > 0) {
         if (!(p.pool = hts_tpool_init(ga.nthreads))) {
             fprintf(samtools_stderr, "Error creating thread pool\n");
             ret = 1;
@@ -1238,13 +1298,20 @@ int main_samview(int argc, char *argv[])
     // Initialize BAM/CRAM index
     char **regs = NULL;
     int nregs = 0;
-    if ( has_index_file && optind < argc - 2 ) regs = &argv[optind+2], nregs = argc - optind - 2, settings.fn_idx_in = argv[optind+1];
-    else if ( !has_index_file && optind < argc - 1 ) regs = &argv[optind+1], nregs = argc - optind - 1;
-    else if ( has_index_file )
-    {
+    if ( has_index_file && optind <= argc - 2 ) {
+        regs = optind < argc-2 ? &argv[optind+2] : NULL;
+        nregs = argc - optind - 2;
+        settings.fn_idx_in = argv[optind+1];
+    } else if (!has_index_file && optind < argc - 1 ) {
+        regs = &argv[optind+1];
+        nregs = argc - optind - 1;
+    } else if ( has_index_file && argc-optind < 2) {
         print_error("view", "Incorrect number of arguments for -X option. Aborting.");
         return 1;
     }
+    if (regs)
+        settings.count_rf |= SAM_POS | SAM_RNAME | SAM_CIGAR;
+
     if ( settings.fn_idx_in || nregs || settings.multi_region )
     {
         settings.hts_idx = settings.fn_idx_in ? sam_index_load2(settings.in, settings.fn_in, settings.fn_idx_in) : sam_index_load(settings.in, settings.fn_in);
@@ -1255,6 +1322,10 @@ int main_samview(int argc, char *argv[])
         }
     }
 
+    if (settings.is_count)
+        // Won't fail, but also wouldn't matter if it did
+        hts_set_opt(settings.in, CRAM_OPT_REQUIRED_FIELDS, settings.count_rf);
+
     if ( settings.fetch_pairs )
     {
         hts_itr_multi_t *iter = multi_region_init(&settings, regs, nregs);
@@ -1267,8 +1338,8 @@ int main_samview(int argc, char *argv[])
         ret = iter ? multi_region_view(&settings, iter) : 1;
         if (ret) goto view_end;
     }
-    else if ( !settings.hts_idx )   // stream through the entire file
-    {
+    else if ( !settings.hts_idx || optind+1 >= argc-has_index_file ) {
+        // stream through the entire file
         ret = stream_view(&settings);
         if (ret) goto view_end;
     } else {   // retrieve alignments in specified regions
@@ -1354,10 +1425,7 @@ view_end:
         free(settings.fn_un_out_idx);
     free(arg_list);
 
-    if (settings.keep_tag)
-        kh_destroy(aux_exists, settings.keep_tag);
-    if (settings.remove_tag)
-        kh_destroy(aux_exists, settings.remove_tag);
+    aux_list_free(&settings);
 
     return ret;
 }
@@ -1419,6 +1487,8 @@ static int usage(FILE *fp, int exit_status, int is_long_help)
 "               Comma-separated read tags to preserve (repeatable) [null].\n"
 "               Equivalent to \"-x ^STR\"\n"
 "  -B, --remove-B             Collapse the backward CIGAR operation\n"
+"  -z, --sanitize FLAGS       Perform sanitity checking and fixing on records.\n"
+"                             FLAGS is comma separated (see manual). [off]\n"
 "\n"
 "General options:\n"
 "  -?, --help   Print long help, including note about region specification\n"
