@@ -1,11 +1,4 @@
-import os
-import glob
-import sys
-import unittest
 import pysam
-import shutil
-import gzip
-import subprocess
 import pytest
 
 try:
@@ -13,7 +6,7 @@ try:
 except ImportError:
     Path = None
 
-from TestUtils import get_temp_filename, check_lines_equal, load_and_convert, make_data_files, CBCF_DATADIR, get_temp_context
+from TestUtils import make_data_files, CBCF_DATADIR
 
 
 def setUpModule():
@@ -23,8 +16,7 @@ def setUpModule():
 @pytest.fixture
 def vcf_header():
     vcf_header = pysam.VariantHeader()
-    vcf_header.add_sample("sample1")
-    vcf_header.add_sample("sample2")
+    vcf_header.add_samples("sample1", "sample2")
     vcf_header.contigs.add("1")
     return vcf_header
 
@@ -74,3 +66,30 @@ def test_unicode_annotation_can_be_added(vcf_header):
     assert str(record)[:-1].split("\t")[-2:] == [
         "anno1",
         "Friedrich-Alexander-Universit\u00E4t_Erlangen-N\u00FCrnberg"]
+
+def test_set_sample_alleles(vcf_header):
+    vcf_header.formats.add('GT',1,'String',"Genotype") # id, number, type, description
+    record = vcf_header.new_record(
+        contig="1",
+        start=20,
+        stop=21,
+        alleles=('A','T')
+        )
+
+    record.samples['sample1'].alleles = ('T', 'A')
+    assert record.samples['sample1'].alleles  == ('T','A')
+
+    # Empty record:
+    record.samples['sample1'].alleles = (None, )
+    assert record.samples['sample1'].alleles   == tuple()
+    record.samples['sample1'].alleles = None
+    assert record.samples['sample1'].alleles   == tuple()
+    record.samples['sample1'].alleles = tuple()
+    assert record.samples['sample1'].alleles   == tuple()
+
+    # check error conditions:
+    with pytest.raises(ValueError, match='One or more of the supplied sample alleles are not defined'):
+        record.samples['sample1'].alleles = ('C', 'A')
+
+    with pytest.raises(ValueError, match='Use .allele_indices to set integer allele indices'):
+        record.samples['sample1'].alleles = (1, 0)
